@@ -13,7 +13,7 @@ from playwright.async_api import (
 
 from nonebot_plugin_htmlrender.config import plugin_config
 from nonebot_plugin_htmlrender.install import install_browser
-from nonebot_plugin_htmlrender.utils import proxy_settings, suppress_and_log
+from nonebot_plugin_htmlrender.utils import proxy_settings, suppress_and_log, with_lock
 
 _browser: Optional[Browser] = None
 _playwright: Optional[Playwright] = None
@@ -53,8 +53,6 @@ async def init_browser(**kwargs) -> Browser:
     Raises:
         RuntimeError: 如果浏览器无法启动或安装失败。
     """
-    await install_browser()  # update playwright when start
-    await check_playwright_env()
     return await start_browser(**kwargs)
 
 
@@ -72,12 +70,11 @@ async def get_new_page(device_scale_factor: float = 2, **kwargs) -> AsyncIterato
     """
     ctx = await get_browser()
     page = await ctx.new_page(device_scale_factor=device_scale_factor, **kwargs)
-    try:
+    async with page:
         yield page
-    finally:
-        await page.close()
 
 
+@with_lock
 async def get_browser(**kwargs) -> Browser:
     """
     获取浏览器实例。
@@ -117,6 +114,32 @@ async def _connect_via_cdp(**kwargs) -> Browser:
         raise RuntimeError("Playwright 未初始化")
 
 
+async def _connect(browser_type: str, **kwargs) -> Browser:
+    """
+    通过 Playwright 协议连接浏览器。
+
+    Args:
+        browser_type (str): 浏览器类型。
+        **kwargs: 传递给`playwright.connect`的关键字参数。
+
+    Returns:
+        Browser: 启动的浏览器实例。
+
+    Raises:
+        RuntimeError: 如果 Playwright 未初始化。
+    """
+    _browser_cls: BrowserType = getattr(_playwright, browser_type)
+    kwargs["ws_endpoint"] = plugin_config.htmlrender_connect
+    logger.info(
+        f"正在使用 Playwright 协议连接 {browser_type}({plugin_config.htmlrender_connect})"
+    )
+    if _playwright is not None:
+        return await _browser_cls.connect(**kwargs)
+    else:
+        raise RuntimeError("Playwright 未初始化")
+
+
+@with_lock
 async def start_browser(**kwargs) -> Browser:
     """
     启动 Playwright 浏览器实例。
@@ -128,24 +151,35 @@ async def start_browser(**kwargs) -> Browser:
         Browser: 启动的浏览器实例。
     """
     global _browser, _playwright
+
+    await shutdown_browser()
     _playwright = await async_playwright().start()
 
     if (
         plugin_config.htmlrender_browser == "chromium"
         and plugin_config.htmlrender_connect_over_cdp
     ):
-        return await _connect_via_cdp(**kwargs)
+        _browser = await _connect_via_cdp(**kwargs)
+    elif plugin_config.htmlrender_connect:
+        _browser = await _connect(plugin_config.htmlrender_browser, **kwargs)
+    else:
+        if plugin_config.htmlrender_browser_channel:
+            kwargs["channel"] = plugin_config.htmlrender_browser_channel
 
-    if plugin_config.htmlrender_browser_channel:
-        kwargs["channel"] = plugin_config.htmlrender_browser_channel
+        if plugin_config.htmlrender_proxy_host:
+            kwargs["proxy"] = proxy_settings(plugin_config.htmlrender_proxy_host)
 
-    if plugin_config.htmlrender_proxy_host:
-        kwargs["proxy"] = proxy_settings(plugin_config.htmlrender_proxy_host)
+        if plugin_config.htmlrender_browser_executable_path:
+            kwargs["executable_path"] = plugin_config.htmlrender_browser_executable_path
+        else:
+            try:
+                await check_playwright_env()
+            except RuntimeError:
+                await install_browser()
+                await check_playwright_env()
 
-    if plugin_config.htmlrender_browser_executable_path:
-        kwargs["executable_path"] = plugin_config.htmlrender_browser_executable_path
+        _browser = await _launch(plugin_config.htmlrender_browser, **kwargs)
 
-    _browser = await _launch(plugin_config.htmlrender_browser, **kwargs)
     return _browser
 
 
@@ -164,7 +198,7 @@ async def check_playwright_env():
     logger.info("Checking Playwright environment...")
     try:
         async with async_playwright() as p:
-            await p.chromium.launch()
+            await getattr(p, plugin_config.htmlrender_browser).launch()
     except Exception as e:
         raise RuntimeError(
             "Playwright environment is not set up correctly. "
